import os
import subprocess

scriptname_parsefile = "parsetools/parsefile.jl"
scriptname_parsefunc = "parsetools/parsefunc.jl"


class JuliaParser:
    cached_files = {}

    def parsefile(self, sourcepath):
        if not os.path.exists(sourcepath):
            raise ValueError("Can't parse julia file: " + sourcepath)
        directory = os.path.dirname(os.path.realpath(__file__))
        scriptpath = os.path.join(directory, scriptname_parsefile)
        p = subprocess.Popen(["julia", scriptpath, sourcepath],
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                             universal_newlines=True)
        (buf, err) = p.communicate()
        if err:
            raise Exception(err)
        d = eval(buf)
        self.cached_files[sourcepath] = d
        return d

    def functions_from_file(self, sourcepath):
        if sourcepath in self.cached_files:
            d = self.cached_files[sourcepath]
        else:
            d = self.parsefile(sourcepath)
        return d

    def function_from_file(self, sourcepath, functionname):
        if sourcepath in self.cached_files:
            d = self.cached_files[sourcepath]
        else:
            d = self.parsefile(sourcepath)
        for func in d:
            if func["qualifier"]+"."+func["name"] == functionname:
                return func
        for func in d:
            if func["name"] == functionname:
                return func
        errortext = "Function {} not found in: {}"
        raise ValueError(errortext.format(functionname, sourcepath))

    def parsefunction(self, functionstring):
        directory = os.path.dirname(os.path.realpath(__file__))
        scriptpath = os.path.join(directory, scriptname_parsefunc)
        p = subprocess.Popen(["julia", scriptpath, functionstring],
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                             universal_newlines=True)
        (buf, err) = p.communicate()
        if err:
            raise Exception(err)
        function = eval(buf)
        return function



import os
import docutils
import sphinx.ext.napoleon

from . import juliadomain


class FunctionDirective(docutils.parsers.rst.Directive):
    has_content = False
    required_arguments = 2

    def __init__(self, name, arguments, options, content, lineno,
                content_offset, block_text, state, state_machine):
        docutils.parsers.rst.Directive.__init__(
                self, name, arguments, options, content, lineno,
                content_offset, block_text, state, state_machine)
        self._directive = juliadomain.JuliaFunction(
                "function", ["<auto>"], {}, docutils.statemachine.ViewList(),
                lineno, content_offset, block_text, state, state_machine)

    def run(self):
        env = self.state.document.settings.env
        sourcedir = env.app.config.juliaautodoc_basedir
        sourcename = self.arguments[0]
        sourcepath = os.path.join(sourcedir, sourcename)
        functionname = self.arguments[1]
        function = env.juliaparser.function_from_file(sourcepath, functionname)
        docstringlines = function["docstring"].split("\n")
        env.app.emit('autodoc-process-docstring',
                                  "function", function["name"], None,
                                  {}, docstringlines)
        function["docstring"] = "\n".join(docstringlines)
        self._directive.function = function
        return self._directive.run()


# Ugly hack to use :kwparam: in napoleon docstring parsing.
def _parse_keyword_arguments_section(self, section):
    fields = self._consume_fields()
    if self._config.napoleon_use_param:
        lines = []
        for _name, _type, _desc in fields:
            field = ':kwparam %s: ' % _name
            lines.extend(self._format_block(field, _desc))
            if _type:
                lines.append(':kwtype %s: %s' % (_name, _type))
        return lines + ['']
    else:
        return self._format_fields('Keyword Parameters', fields)

sphinx.ext.napoleon.docstring.GoogleDocstring._parse_keyword_arguments_section = _parse_keyword_arguments_section


def setup(app):
    app.add_config_value('juliaautodoc_basedir', '..', 'html')
    app.add_directive('jl:autofunction', FunctionDirective)
    app.add_event('autodoc-process-docstring')
    app.add_event('autodoc-skip-member')
    return {'version': '0.1'}



# -*- coding: utf-8 -*-
"""
sphinx.domains.julia
~~~~~~~~~~~~~~~~~~~~

The Julia domain.

:copyright: Copyright 2015 by Sebastian Krämer
:license: BSD, see LICENSE for details.
"""

from six import iteritems

from docutils import nodes
from docutils.parsers.rst import directives
from docutils.statemachine import ViewList

import sphinx
from sphinx import addnodes
from sphinx.roles import XRefRole
from sphinx.locale import l_, _
from sphinx.domains import Domain, ObjType, Index
from sphinx.directives import ObjectDescription
from sphinx.util.nodes import make_refnode
from sphinx.util.compat import Directive
from sphinx.util.docfields import Field, GroupedField, TypedField

from . import parser


class desc_keyparameter(nodes.Part, nodes.Inline, nodes.TextElement):
    """
    Node for a keyword parameter.
    """


class JuliaFunction(sphinx.directives.ObjectDescription):
    """
    Julia function directive.
    """

    option_spec = {
        # 'noindex': directives.flag,
        # 'module': directives.unchanged,
    }

    doc_field_types = [
        TypedField('parameter', label=l_('Parameters'),
                   names=('param', 'parameter', 'arg', 'argument'),
                   typerolename='obj', typenames=('paramtype', 'type'),
                   can_collapse=True),
        TypedField('kwparam', label=l_('Keyword Parameters'),
                   names=('kwparam', 'kwparameter', 'kwarg', 'kwargument'),
                   typerolename='obj', typenames=('kwparamtype', 'kwtype'),
                   can_collapse=True),
        GroupedField('exceptions', label=l_('Exceptions'), rolename='exc',
                     names=('raises', 'raise', 'exception', 'except'),
                     can_collapse=True),
        Field('returnvalue', label=l_('Returns'), has_arg=False,
              names=('returns', 'return')),
        Field('returntype', label=l_('Return type'), has_arg=False,
              names=('rtype',), bodyrolename='obj'),
    ]

    def needs_arglist(self):
        return True

    def handle_signature(self, sig, signode):
        """
        Transform a Julia function signature into RST nodes.
        """
        env = self.state.document.settings.env

        # Directive was invoked from autodoc
        if sig == "<auto>":
            function = self.function
            content = ViewList(function["docstring"].split("\n"))
            self.content = content
        # Directive was invoked normally
        else:
            functionstring = "function " + sig + "end"
            function = env.juliaparser.parsefunction(functionstring)
            self.function = function

        # Full function name: name{T}
        # modules are not handled at the moment
        fullname = ""
        if env.config.julia_signature_show_qualifier and function["qualifier"]:
            fullname += function["qualifier"] + "."
        fullname +=  function["name"]
        if function["templateparameters"]:
            fullname += "{" + ",".join(function["templateparameters"]) + "}"

        # Create RST noded
        signode['fullname'] = fullname
        signode += sphinx.addnodes.desc_name(fullname, fullname)
        paramlist = sphinx.addnodes.desc_parameterlist()
        signode += paramlist
        for arg in function["arguments"]:
            x = arg["name"]
            if env.config.julia_signature_show_type and arg["type"]:
                x += "::" + arg["type"]
            if env.config.julia_signature_show_default and arg["value"]:
                x += "=" + arg["value"]
            paramlist += sphinx.addnodes.desc_parameter(x, x)
        for arg in function["kwarguments"]:
            x = arg["name"]
            if env.config.julia_signature_show_type and arg["type"]:
                x += "::" + arg["type"]
            if env.config.julia_signature_show_default and arg["value"]:
                x += "=" + arg["value"]
            paramlist += desc_keyparameter(x, x)
        return fullname

    def before_content(self):
        env = self.state.document.settings.env
        if env.config.julia_docstring_show_type:
            l = []
            for arg in self.function["arguments"]:
                if arg["type"]:
                    l.append(":type {name}: :class:`{type}`\n".format(**arg))
            for arg in self.function["kwarguments"]:
                if arg["type"]:
                    l.append(":kwtype {name}: :class:`{type}`\n".format(**arg))
            self.content.append(ViewList(l))


class JuliaDomain(sphinx.domains.Domain):
    """
    Julia language domain.
    """
    name = 'jl'
    label = 'Julia'
    object_types = {
        'function': sphinx.domains.ObjType(l_('function'), 'func', 'obj'),
        # 'global':          ObjType(l_('global variable'),  'global', 'obj'),
        # 'class':            ObjType(l_('class'),            'class', 'obj'),
        # 'exception':       ObjType(l_('exception'),        'exc', 'obj'),
        # 'attribute':       ObjType(l_('attribute'),        'attr', 'obj'),
        # 'const':           ObjType(l_('const'),            'const', 'obj'),
        # 'module':          ObjType(l_('module'),           'mod', 'obj'),
    }

    directives = {
        'function':        JuliaFunction,
        # 'global':          JuliaGloballevel,
        # 'const':           JuliaEverywhere,
        # 'class':            JuliaClasslike,
        # 'exception':       JuliaClasslike,
        # 'attribute':     JuliaAttribute,
        # 'module':          JuliaModule,
    }

    roles = {
        # 'func':   JuliaXRefRole(fix_parens=False),
        # 'global': JuliaXRefRole(),
        # 'class':  JuliaXRefRole(),
        # 'exc':    JuliaXRefRole(),
        # 'attr':   JuliaXRefRole(),
        # 'const':  JuliaXRefRole(),
        # 'mod':    JuliaXRefRole(),
        # 'obj':    JuliaXRefRole(),
    }

    initial_data = {
        'objects': {},  # fullname -> docname, objtype
        'modules': {},  # modname -> docname, synopsis, platform, deprecated
    }
    indices = [
        # JuliaModuleIndex,
    ]

def visit_desc_keyparameter(self, node):
    if self.first_kwordparam:
        self.first_kwordparam = False
        self.body.append('; ')
    else:
        self.body.append(', ')
    if not node.hasattr('noemph'):
        self.body.append('<em>')


def depart_desc_keyparameter(self, node):
    if not node.hasattr('noemph'):
        self.body.append('</em>')


def update_builder(app):
    app.env.juliaparser = parser.JuliaParser()
    translator = app.builder.translator_class
    translator.first_kwordparam = True
    _visit_desc_parameterlist = translator.visit_desc_parameterlist
    def visit_desc_parameterlist(self, node):
        self.first_kwordparam = True
        _visit_desc_parameterlist(self, node)
    translator.visit_desc_parameterlist = visit_desc_parameterlist


def setup(app):
    app.add_config_value('julia_signature_show_qualifier', True, 'html')
    app.add_config_value('julia_signature_show_type', True, 'html')
    app.add_config_value('julia_signature_show_default', True, 'html')
    app.add_config_value('julia_docstring_show_type', True, 'html')

    app.add_node(desc_keyparameter,
                html=(visit_desc_keyparameter, depart_desc_keyparameter)
                )
    app.connect('builder-inited', update_builder)
    app.add_domain(JuliaDomain)



#!/usr/bin/env python

from distutils.core import setup

setup(name='Sphinx-Julia',
      version='0.1',
      description='Sphinx extensions for Julia',
      author='Sebastian Kraemer',
      author_email='basti.kr@gmail.com',
      url='https://github.com/bastikr/sphinx-julia',
      license='MIT',
      packages=['sphinxjulia'],
      package_data={'sphinxjulia': ['parsetools/*.jl']},
     )